@@ -40,10 +40,17 @@
 #include "ils.h"
 #include "urlhandler.h"
 #include "history-combo.h"
+#include "lid.h"
 
 #include <esd.h>
 #include <gconf/gconf-client.h>
 
+
+#ifndef DISABLE_GNOME
+#include <gnome.h>
+#endif
+
+
 /* Declarations */
 GtkWidget *gm;
 GnomeMeeting *MyApp;	
@@ -64,9 +71,8 @@ GnomeMeeting::GnomeMeeting ()
   gw = gnomemeeting_get_main_window (gm);
   lw = gnomemeeting_get_ldap_window (gm);
 
-  MyApp = (this);
-
   endpoint = new GMH323EndPoint ();
+  MyApp = (this);
 
   call_number = 0;
 }
@@ -74,7 +80,6 @@ GnomeMeeting::GnomeMeeting ()
 
 GnomeMeeting::~GnomeMeeting()
 {
-  delete (endpoint);
 }
 
 
@@ -90,31 +95,22 @@ GnomeMeeting::Connect()
   connection = endpoint->GetCurrentConnection ();
   
   gnomemeeting_threads_enter ();
-  gnome_appbar_clear_stack (GNOME_APPBAR (gw->statusbar));
+  gnomemeeting_statusbar_push  (gw->statusbar, NULL);
   call_address = (PString) gtk_entry_get_text 
           (GTK_ENTRY (GTK_COMBO (gw->combo)->entry));  
   gnomemeeting_threads_leave ();
 
 
   /* If connection, then answer it */
-  if (connection != NULL) 
-  {
-    
-#ifdef HAS_IXJ
-      OpalLineInterfaceDevice *lid = NULL;
-      lid = endpoint->GetLidDevice ();
-      if (lid)
-	lid->StopTone (0);
-#endif
+  if (connection != NULL) {
 
-      endpoint->SetCallingState (2);
-      connection->AnsweringCall (H323Connection::AnswerCallNow);
-      
-      gnomemeeting_threads_enter ();
-      gnomemeeting_log_insert (gw->history_text_view,
-			       _("Answering incoming call"));
-      connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 1);
-      gnomemeeting_threads_leave ();
+    connection->AnsweringCall (H323Connection::AnswerCallNow);
+    
+    gnomemeeting_threads_enter ();
+    gnomemeeting_log_insert (gw->history_text_view,
+			     _("Answering incoming call"));
+    connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 1);
+    gnomemeeting_threads_leave ();
   }
   else 
   {
@@ -138,14 +134,17 @@ GnomeMeeting::Connect()
 
       gnomemeeting_threads_enter ();
       gw->progress_timeout =
-	gtk_timeout_add (20, gnomemeeting_window_appbar_update, 
-			 gw->statusbar);
-      gtk_widget_show (GTK_WIDGET (gnome_appbar_get_progress (GNOME_APPBAR(gw->statusbar))));
+	gtk_timeout_add (5, gnomemeeting_window_appbar_update, 
+			 gw->progressbar);
+      gtk_widget_show (gw->progressbar);
       gnomemeeting_threads_leave ();
 
 #ifdef HAS_IXJ
       OpalLineInterfaceDevice *lid = NULL;
-      lid = endpoint->GetLidDevice ();
+      GMLid *lid_thread = NULL;
+      lid_thread = endpoint->GetLidThread ();
+      if (lid_thread)
+	lid = lid_thread->GetLidDevice ();
       if (lid)
 	lid->PlayTone (0, OpalLineInterfaceDevice::RingTone);
 #endif
@@ -179,9 +178,9 @@ void GnomeMeeting::Disconnect (H323Connection::CallEndReason reason)
 
     gtk_timeout_remove (gw->progress_timeout);
     gw->progress_timeout = 0;
-    gtk_widget_hide (GTK_WIDGET (gnome_appbar_get_progress (GNOME_APPBAR (gw->statusbar))));
+    gtk_widget_hide (gw->progressbar);
   }
-  gnome_appbar_clear_stack (GNOME_APPBAR (gw->statusbar));
+  gnomemeeting_statusbar_push (gw->statusbar, NULL);
 
   gnomemeeting_threads_leave ();
 
