@@ -28,20 +28,28 @@
  */
 
 	
-#include <sys/time.h>
-#include <ptlib.h>
-#include <ldap.h>
-
 #include "../config.h"
 
 #include "ils.h"
+#include "ldap_window.h"
 #include "gnomemeeting.h"
 #include "common.h"
 #include "misc.h"
 #include "stock-icons.h"
 #include "dialog.h"
+#include "misc.h"
 #include "xdap.h"
 
+#include <sys/time.h>
+#include <ptlib.h>
+#include <ldap.h>
+
+
+#ifndef DISABLE_GNOME
+#include <gnome.h>
+#endif
+
+
 #include "../pixmaps/inlines.h"
 
 
@@ -147,11 +155,6 @@ GMILSClient::~GMILSClient ()
   running = 0;
 
   quit_mutex.Wait ();
-
-  /* After the thread has stopped,
-     we unregister if we were registered */
-  if (registered == 1)
-    Register (0);
 }
 
 
@@ -164,17 +167,20 @@ void GMILSClient::Main ()
     /* The most important operation is to unregister */
     if (has_to_unregister == 1) {
      
-      Register (0);
+      if (registered)
+	Register (0);
     }
 
     if (has_to_register == 1) {
 
-      Register (1);
+      if (!registered)
+	Register (1);
     }
 
     if (has_to_modify == 1) {
 
-      Register (2);
+      if (registered)
+	Register (2);
     }
 
 
@@ -192,9 +198,15 @@ void GMILSClient::Main ()
 	starttime = PTime ();
       }
 
-    Current ()->Sleep (500);
+    Current ()->Sleep (100);
   }
 
+  /* After the thread has stopped,
+     we unregister if we were registered */
+  if (registered == 1)
+    Register (0);
+
+
   quit_mutex.Signal ();
 }
 
@@ -204,8 +216,6 @@ BOOL GMILSClient::CheckFieldsConfig ()
   gchar *firstname = NULL;
   gchar *surname = NULL;
   gchar *mail = NULL;
-  gchar *comment = NULL;
-  gchar *location = NULL;
   bool registering = TRUE;
   bool no_error = TRUE;
 
@@ -221,14 +231,6 @@ BOOL GMILSClient::CheckFieldsConfig ()
     gconf_client_get_string (GCONF_CLIENT (client),
 			     "/apps/gnomemeeting/personal_data/mail", 
 			     NULL);
-  comment =  
-    gconf_client_get_string (GCONF_CLIENT (client),
-			     "/apps/gnomemeeting/personal_data/comment", 
-			     NULL);
-  location =  
-    gconf_client_get_string (GCONF_CLIENT (client),
-			     "/apps/gnomemeeting/personal_data/location", 
-			     NULL);
   
   registering =
     gconf_client_get_bool (GCONF_CLIENT (client),
@@ -240,12 +242,10 @@ BOOL GMILSClient::CheckFieldsConfig ()
 
     if ((firstname == NULL) || (!strcmp (firstname, ""))
 	|| (surname == NULL) || (!strcmp (surname, ""))
-	|| (comment == NULL) || (!strcmp (comment, ""))
-	|| (location == NULL) || (!strcmp (location, ""))
 	|| (mail == NULL) || (!strcmp (mail, ""))) {
       
       /* No need to display that for unregistering */
-      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Please provide your first name, last name, comment, e-mail and location details in the Personal Data section in order to be able to register to the XDAP server."));
+      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Please provide your first name, last name, and e-mail in the Personal Data section in order to be able to register to the XDAP server."));
       
       no_error = FALSE;
     }
@@ -258,8 +258,6 @@ BOOL GMILSClient::CheckFieldsConfig ()
   g_free (firstname);
   g_free (surname);
   g_free (mail);
-  g_free (comment);
-  g_free (location);			       
       
   return no_error;
 }
@@ -348,7 +346,7 @@ BOOL GMILSClient::Register (int reg)
     msg = g_strdup_printf (_("Contacting %s..."), ldap_server);
 
     if (reg != 2)
-      gnomemeeting_statusbar_flash (gm, msg);
+      gnomemeeting_statusbar_flash (gw->statusbar, msg);
     gnomemeeting_log_insert (gw->history_text_view, msg);    
     g_free (msg);
     gnomemeeting_threads_leave ();
@@ -483,7 +481,7 @@ BOOL GMILSClient::Register (int reg)
       }
 
       if (reg != 2)
-	gnomemeeting_statusbar_flash (gm, msg);
+	gnomemeeting_statusbar_flash (gw->statusbar, msg);
       gnomemeeting_log_insert (gw->history_text_view, msg);
       g_free (msg);
 
@@ -507,7 +505,7 @@ BOOL GMILSClient::Register (int reg)
 			     ldap_server);
     
     gnomemeeting_log_insert (gw->history_text_view, msg);
-    gnomemeeting_statusbar_flash (gm, msg);
+    gnomemeeting_statusbar_flash (gw->statusbar, msg);
     gnomemeeting_error_dialog (GTK_WINDOW (gm), msg);
     g_free (msg);
 
@@ -526,8 +524,9 @@ BOOL GMILSClient::Register (int reg)
 
 gchar *GMILSClient::Search (gchar *ldap_server, gchar *ldap_port, gchar *mail)
 {
-  char *attrs [] = { "rfc822mailbox", "sipaddress", "sport", NULL };
+  char *attrs [] = { "rfc822mailbox", "sappid", "sipaddress", "sport", NULL };
   char **ldv = NULL;
+ 
   unsigned long int nmip = 0;
   int part1;
   int part2;
@@ -539,6 +538,7 @@ gchar *GMILSClient::Search (gchar *ldap_server, gchar *ldap_port, gchar *mail)
   bool no_error = TRUE;
   gchar *ip = NULL;
   gchar *cn = NULL;
+  gchar *app = NULL;
 
   LDAPMessage *res = NULL, *e = NULL;
 
@@ -593,20 +593,37 @@ gchar *GMILSClient::Search (gchar *ldap_server, gchar *ldap_port, gchar *mail)
     if (e) {
       
       ldv = ldap_get_values (ldap_search_connection, e, "sipaddress");
-      
       if ((ldv != NULL)&&(ldv [0] != NULL)) {
 	
 	nmip = strtoul (ldv [0], NULL, 10);
 	ldap_value_free (ldv);
       }
 
-      ldv = ldap_get_values (ldap_search_connection, e, "sport");
 
+      ldv = ldap_get_values (ldap_search_connection, e, "sappid");
+      if ((ldv != NULL)&&(ldv [0] != NULL)) {
+	
+	app = g_strdup (ldv [0]);
+	ldap_value_free (ldv);
+      }
+
+
+      ldv = ldap_get_values (ldap_search_connection, e, "sport");
       if ((ldv != NULL)&&(ldv [0] != NULL)) {
 	
 	port = atoi (ldv [0]);
+	
+	/* Ugly hack because Netmeeting sucks, it registers
+	   random ports to ILS, but it only supports 1720 */
+	if (app&&!strcmp (app, "ms-netmeeting")&&port == 1503) {
+
+	  port = 1720;
+	  g_free (app);
+	}
+
 	ldap_value_free (ldv);
       }
+
     }
     
     part1 = (int) (nmip/(256*256*256));
@@ -634,6 +651,7 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
 {
   xmlEntityPtr entity;
   xmlChar *entval;
+  xmlChar *encentval;
   xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
 
   gchar *firstname = NULL;
@@ -646,7 +664,8 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
   gchar *ip = NULL;
   gchar *port = NULL;
   gchar *ilsa32964638 = NULL;
-
+  gchar *ilsa39321630 = NULL;
+  
   unsigned long int sip = 0;
 
   GConfClient *client = gconf_client_get_default ();
@@ -671,11 +690,15 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
     gconf_client_get_string (GCONF_CLIENT (client),
 			     "/apps/gnomemeeting/personal_data/comment", 
 			     NULL);
+  if (!comment || !strcmp (comment, ""))
+    comment = g_strdup (" ");
 
   location =  
     gconf_client_get_string (GCONF_CLIENT (client),
 			     "/apps/gnomemeeting/personal_data/location", 
 			     NULL);
+  if (!location || !strcmp (location, ""))
+    location = g_strdup (" ");
 
   port = 
     g_strdup_printf ("%d", 
@@ -695,6 +718,11 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
   else
     busy = g_strdup ("0");
 
+  if (gconf_client_get_bool (client, LDAP_KEY "visible", NULL))
+    ilsa39321630 = g_strdup ("1");
+  else
+    ilsa39321630 = g_strdup ("8");
+  
   if (gconf_client_get_bool (client, "/apps/gnomemeeting/video_settings/enable_video_transmission", NULL))
     ilsa32964638 = g_strdup ("1");
   else
@@ -724,7 +752,7 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
   else if (!strcmp ((char *) name, "port"))
     entval = xmlStrdup (BAD_CAST port);
   else if (!strcmp ((char *) name, "decip"))
-    entval = xmlStrdup (BAD_CAST "1234567890");
+    entval = xmlStrdup (BAD_CAST ip);
   else if (!strcmp ((char *) name, "email"))
     entval = xmlStrdup (BAD_CAST mail);
   else if (!strcmp ((char *) name, "givenname"))
@@ -733,10 +761,15 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
     entval = xmlStrdup (BAD_CAST surname);
   else if (!strcmp ((char *) name, "sappid"))
     entval = xmlStrdup (BAD_CAST "GnomeMeeting");
+  else if (!strcmp ((char *) name, "ilsa39321630"))
+      entval = xmlStrdup (BAD_CAST ilsa39321630);
   else if (!strcmp ((char *) name, "ilsa26279966"))
     entval = xmlStrdup (BAD_CAST version);
   else
     return 0;
+
+  encentval = xmlEncodeSpecialChars(((xmlParserCtxtPtr) ctx)->myDoc, entval);
+
   if (!ctxt->myDoc->intSubset) {
     (void) xmlCreateIntSubset (ctxt->myDoc, BAD_CAST "ils",
 			       BAD_CAST "SYSTEM",
@@ -749,14 +782,17 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
 
   entity = xmlGetDocEntity(((xmlParserCtxtPtr) ctx)->myDoc, name);
   if (entity) {
-    if (strcmp((char *)entity->content, (char *)entval))
+    if (strcmp((char *)entity->content, (char *)encentval))
       fprintf(stderr,
 	      "Warning: New entity value will be ignored %s %s %s\n",
-	      name, entval, entity->content);
+	      name, encentval, entity->content);
   } else
     entity = xmlAddDocEntity (((xmlParserCtxtPtr) ctx)->myDoc,
-			      name, XML_INTERNAL_GENERAL_ENTITY, 0, 0, entval);
+			      name, XML_INTERNAL_GENERAL_ENTITY, 0, 0, 
+			      encentval);
 
+  xmlFree(entval);
+  xmlFree(encentval);
 
   g_free (firstname);
   g_free (surname);
@@ -768,7 +804,8 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
   g_free (ip);
   g_free (port);
   g_free (ilsa32964638);
-
+  g_free (ilsa39321630);
+  
   return entity;
 }
 
@@ -817,7 +854,7 @@ void GMILSBrowser::Main ()
   char *attrs [] = { "surname", "givenname", "comment", "location", 
 		     "rfc822mailbox", "sipaddress", "ilsa32833566", 
 		     "ilsa32964638", "ilsa26279966", "ilsa26214430", 
-		     "sport", "sappid", 
+		     "sport", "sappid", "xstatus", 
 		     NULL };
 
   char **ldv;
@@ -843,10 +880,11 @@ void GMILSBrowser::Main ()
   gchar *msg = NULL;
   gchar *color = NULL;
   gchar *filter = NULL;
-
-  GtkWidget *label = NULL;
-  GtkWidget *tab_label = NULL;
-  GtkWidget *close_button = NULL;
+  gchar *name = NULL;
+  gchar *callto = NULL;
+  gchar **num_users = NULL;
+  
+  GtkWidget *statusbar = NULL;
   GtkListStore *xdap_users_list_store = NULL;
   GtkWidget *xdap_users_tree_view = NULL;
   GdkPixbuf *status_icon = NULL;
@@ -862,8 +900,37 @@ void GMILSBrowser::Main ()
   msg = g_strdup_printf (_("Contacting %s..."), ldap_server);
 
   gnomemeeting_threads_enter ();
+  /* Check if the page with the ldap_server we are browsing still exists */
+  while ((page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (lw->notebook), 
+					    cj))) {
+    
+    text_label = 
+      (gchar *) g_object_get_data (G_OBJECT (page), "contact_section");
+    if (!(strcasecmp (text_label, ldap_server))) {
+      
+      page_exists = 1;
+      break;
+    }
+    
+    cj++;
+  }
+    
+  /* If we found a page, we have the LIST_STORE, if not, we can clean
+     the statusbar */
+  if (page != NULL) {
+    
+    xdap_users_list_store = 
+      GTK_LIST_STORE (g_object_get_data (G_OBJECT (page), "list_store"));
+    xdap_users_tree_view =
+      GTK_WIDGET (g_object_get_data (G_OBJECT (page), "tree_view"));
+    statusbar = 
+      GTK_WIDGET (g_object_get_data (G_OBJECT (page), "statusbar"));
+  }
+  else 
+    return;
+
   gw = gnomemeeting_get_main_window (gm);
-  gnome_appbar_push (GNOME_APPBAR (lw->statusbar), msg);
+  gnomemeeting_statusbar_push (statusbar, msg);
   g_free (msg);
   gnomemeeting_threads_leave ();
 
@@ -908,7 +975,7 @@ void GMILSBrowser::Main ()
     gnomemeeting_threads_enter ();        
     msg = g_strdup_printf (_("Fetching user information from %s."),
 			   ldap_server);
-    gnome_appbar_push (GNOME_APPBAR (lw->statusbar), msg);
+    gnomemeeting_statusbar_push (statusbar, msg);
     gnomemeeting_threads_leave ();
     g_free (msg);
 
@@ -917,6 +984,7 @@ void GMILSBrowser::Main ()
     else
       filter = g_strdup ("(&(cn=%))");
 
+    
     rc = ldap_search_s (ldap_connection, "objectClass=RTPerson", 
 			LDAP_SCOPE_BASE,
 			filter,	attrs, 0, &res); 
@@ -925,49 +993,10 @@ void GMILSBrowser::Main ()
     for (int i = 0 ; i < 9 ; i++)
       datas [i] = NULL;
     
-    gnomemeeting_threads_enter ();
-    /* Check if the page with the ldap_server we are browsing still exists */
-    while ((page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (lw->notebook), 
-					      cj))) {
-     
-      tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (lw->notebook), 
-					      page);
-      label = (GtkWidget *) g_object_get_data (G_OBJECT (tab_label), "label");
-      text_label = (gchar *) gtk_label_get_text (GTK_LABEL (label));
-      if (!(strcasecmp (text_label, ldap_server))) {
-	
-	page_exists = 1;
-	break;
-      }
-      
-      cj++;
-    }
-    
-    /* If we found a page, we have the LIST_STORE, if not, we can clean
-       the statusbar */
-    if (page != NULL) {
-
-      xdap_users_list_store = 
-	GTK_LIST_STORE (g_object_get_data (G_OBJECT (page), "list_store"));
-      xdap_users_tree_view =
-	GTK_WIDGET (g_object_get_data (G_OBJECT (page), "tree_view"));
-    }
-    
-    else
-      gnome_appbar_clear_stack (GNOME_APPBAR (lw->statusbar));
-
-    gnomemeeting_threads_leave ();
-
     
     /* Maybe the user closed the tab while we were waiting */
     if ((xdap_users_list_store != NULL) && (page_exists)) { 
       
-      gnomemeeting_threads_enter ();
-      close_button =
-	GTK_WIDGET (g_object_get_data (G_OBJECT (tab_label), "close_button"));
-      gtk_widget_set_sensitive (close_button, FALSE);
-      gnomemeeting_threads_leave ();
-      
       for (e = ldap_first_entry(ldap_connection, res); 
 	   e != NULL; e = ldap_next_entry(ldap_connection, e)) {
 	
@@ -1010,6 +1039,31 @@ void GMILSBrowser::Main ()
 	    g_strdup (ldv [0]);
 	  ldap_value_free (ldv);
 	}
+
+
+	ldv = ldap_get_values(ldap_connection, e, "sport");
+	if ((ldv != NULL)&&(ldv [0] != NULL)) {
+
+	  port = atoi (ldv [0]);	
+
+	  /* Ugly hack because Netmeeting sucks, it registers
+	     random ports to ILS, but it only supports 1720 */
+	  if (datas [7]&&!strcmp (datas [7], "ms-netmeeting")&&port == 1503)
+	    port = 1720;
+
+	  ldap_value_free (ldv);
+	}
+
+
+	/* Sent one time only by ils.seconix.com only */
+	ldv = ldap_get_values(ldap_connection, e, "xstatus");
+	if ((ldv != NULL)&&(ldv [0] != NULL)) {
+
+	  num_users = g_strsplit (ldv [0], ",", 0);
+
+	  ldap_value_free (ldv);
+	}
+
 	
 	ldv = ldap_get_values(ldap_connection, e, "ilsa26279966");
 	if ((ldv != NULL)&&(ldv [0] != NULL)) {
@@ -1039,12 +1093,6 @@ void GMILSBrowser::Main ()
 	  ldap_value_free (ldv);
 	}
 
-	ldv = ldap_get_values(ldap_connection, e, "sport");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-	
-	  port = atoi (ldv [0]);
-	  ldap_value_free (ldv);
-	}
 	
 	ldv = ldap_get_values(ldap_connection, e, "sipaddress");
 	if ((ldv != NULL)&&(ldv [0] != NULL)) {
@@ -1111,42 +1159,53 @@ void GMILSBrowser::Main ()
 	    (xdap_users_tree_view, 
 	     GM_STOCK_STATUS_OCCUPIED,
 	     GTK_ICON_SIZE_MENU, NULL);
-	  color = g_strdup ("brown");
+	  color = g_strdup ("darkred");
         }
         
 	gnomemeeting_threads_leave ();
 
-
+				
 	/* Check if the window is still present or not */
 	if (lw) {
 	  
 	  gchar *utf8_data [7] = {NULL, NULL, NULL, NULL, NULL, NULL, NULL};
+
 	  
-	  for (int j = 0 ; j < 7 ; j++) 
+	  for (int j = 0 ; j < 7 ; j++) {
 	    
 	    /* Strings are already UTF-8-encoded for LDAP v3,
 	       but Microsoft ILS doesn't take care of that */
-	    if (datas [j + 2])
-	      utf8_data [j] = g_convert (datas [j+2], strlen (datas [j + 2]),
-					 "UTF-8", "ISO-8859-1", NULL, NULL, 
-					 NULL);
+	    if (datas [j + 2]) {
+	      
+	      /* If valid UTF-8, ok */
+	      if (g_utf8_validate (datas [j + 2], -1, NULL))
+		utf8_data [j] = g_strdup (datas [j+2]);
+	      else
+		utf8_data [j] = 
+		  gnomemeeting_from_iso88591_to_utf8 (PString (datas [j+2]));
+	    }
+	  }
+
+	  /* The user name and callto */
+	  name = g_strdup_printf ("%s %s", utf8_data [0], utf8_data [1]);
+	  callto = g_strdup_printf ("callto://%s/%s",
+				    ldap_server, utf8_data [2]);
 
 	  gnomemeeting_threads_enter ();
 
 	  gtk_list_store_append (xdap_users_list_store, &list_iter);
 
 	  gtk_list_store_set (xdap_users_list_store, &list_iter,
-			      COLUMN_STATUS, status_icon,
-			      COLUMN_AUDIO, audio,
-			      COLUMN_VIDEO, video,
-			      COLUMN_FIRSTNAME, utf8_data [0],
-			      COLUMN_LASTNAME, utf8_data [1],
-			      COLUMN_EMAIL, utf8_data [2],
-			      COLUMN_LOCATION, utf8_data [3],
-			      COLUMN_COMMENT, utf8_data [4],
-			      COLUMN_VERSION, utf8_data [5],
-			      COLUMN_IP, utf8_data [6],
-			      COLUMN_COLOR, color,
+			      COLUMN_ILS_STATUS, status_icon,
+			      COLUMN_ILS_AUDIO, audio,
+			      COLUMN_ILS_VIDEO, video,
+			      COLUMN_ILS_NAME, name,
+			      COLUMN_ILS_CALLTO, callto,
+			      COLUMN_ILS_LOCATION, utf8_data [3],
+			      COLUMN_ILS_COMMENT, utf8_data [4],
+			      COLUMN_ILS_VERSION, utf8_data [5],
+			      COLUMN_ILS_IP, utf8_data [6],
+			      COLUMN_ILS_COLOR, color,
 			      -1);
           
 	  gnomemeeting_threads_leave ();
@@ -1159,6 +1218,8 @@ void GMILSBrowser::Main ()
 
 	g_object_unref (status_icon);
 	g_free (color);
+	g_free (name);
+	g_free (callto);
 
 	for (int j = 2 ; j <= 8 ; j++) {
 	  
@@ -1172,10 +1233,18 @@ void GMILSBrowser::Main ()
 
 
       gnomemeeting_threads_enter ();
-      gtk_widget_set_sensitive (close_button, TRUE);
-      msg = g_strdup_printf (_("Search completed: %d user(s) found on %s."),
-			     users_nbr, ldap_server);
-      gnome_appbar_push (GNOME_APPBAR (lw->statusbar), msg); 
+      if (num_users && num_users [1]) {
+	
+	msg =
+	  g_strdup_printf (_("Search completed: %d user(s) listed on a total of %d user(s) from %s."),
+			   users_nbr, PMAX (atoi (num_users [1]), users_nbr), ldap_server);
+	g_strfreev (num_users);
+      }
+      else
+	msg = g_strdup_printf (_("Search completed: %d user(s) found on %s."),
+			       users_nbr, ldap_server);
+	
+      gnomemeeting_statusbar_push (statusbar, msg); 
       g_free (msg);
       gnomemeeting_threads_leave ();
       
@@ -1191,7 +1260,7 @@ void GMILSBrowser::Main ()
     
     gnomemeeting_threads_enter ();
     msg = g_strdup_printf (_("Failed to contact %s."), ldap_server);
-    gnome_appbar_push (GNOME_APPBAR (lw->statusbar), msg);
+    gnomemeeting_statusbar_push (statusbar, msg);
     g_free (msg);
     gnomemeeting_threads_leave ();
   } 
