@@ -37,6 +37,12 @@
 #include "misc.h" 
 #include "stock-icons.h"
 #include "history-combo.h"
+#include "config.h"
+#include "lid.h"
+
+#ifndef DISABLE_GNOME
+#include <gnome.h>
+#endif
 
 
 /* Declarations */
@@ -52,7 +58,6 @@ static void speaker_phone_toggle_changed (GtkToggleButton *, gpointer);
 #endif
 
 static void connect_button_clicked       (GtkToggleButton *, gpointer);
-static void toolbar_toggle_changed       (GtkWidget *, gpointer);
 static void toolbar_button_changed       (GtkWidget *, gpointer);
 static void toolbar_cp_button_changed    (GtkWidget *, gpointer);
 
@@ -75,15 +80,19 @@ static void speaker_phone_toggle_changed (GtkToggleButton *w, gpointer data)
 {
   GmWindow *gw = gnomemeeting_get_main_window (gm);
   OpalLineInterfaceDevice *lid = NULL;
+  GMLid *lid_thread = NULL;
 
-  lid = MyApp->Endpoint ()->GetLidDevice ();
+  lid_thread = MyApp->Endpoint ()->GetLidThread ();
+
+  if (lid_thread)
+    lid = lid_thread->GetLidDevice ();
 
   if (lid) {
 
     if (gtk_toggle_button_get_active (w)) {
       
       gnomemeeting_log_insert (gw->history_text_view,
-			       _("Speakerphone enabled"));
+			       _("Speaker phone enabled"));
       if (MyApp->Endpoint()->GetCallingState () == 0) /* Not in a call */
 	lid->PlayTone (0, OpalLineInterfaceDevice::DialTone);
 
@@ -95,7 +104,7 @@ static void speaker_phone_toggle_changed (GtkToggleButton *w, gpointer data)
     else {
 
       gnomemeeting_log_insert (gw->history_text_view,
-			       _("Speakerphone disabled"));
+			       _("Speaker phone disabled"));
       lid->EnableAudio(0, TRUE);
       lid->StopTone (0);
     }
@@ -126,22 +135,6 @@ static void connect_button_clicked (GtkToggleButton *w, gpointer data)
 }
 
 
-/* DESCRIPTION  :  This callback is called when the user toggles the 
- *                 corresponding component in the toolbar. 
- *                 (See menu_toggle_changed)
- * BEHAVIOR     :  Updates the gconf cache.
- * PRE          :  data is the key.
- */
-static void toolbar_toggle_changed (GtkWidget *w, gpointer data)
-{
-  GConfClient *client = gconf_client_get_default ();
-
-  gconf_client_set_bool (client,
-			 (gchar *) data,
-			 gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)), NULL);
-}
-
-
 /* DESCRIPTION  :  This callback is called when the user presses the control 
  *                 panel button in the toolbar. 
  *                 (See menu_toggle_changed)
@@ -183,50 +176,14 @@ static void toolbar_button_changed (GtkWidget *widget, gpointer data)
 
 
 /* The functions */
-
-void gnomemeeting_init_toolbar ()
+GtkWidget *
+gnomemeeting_init_main_toolbar ()
 {
-  GtkWidget *hbox;
-  GtkWidget *image;
-  GtkWidget *left_toolbar;
-
-  GConfClient *client = gconf_client_get_default ();
+  GtkWidget *hbox = NULL;
+  GtkWidget *image = NULL;
 
   GmWindow *gw = gnomemeeting_get_main_window (gm);
-
-  left_toolbar = gtk_toolbar_new ();
-
-  gtk_toolbar_append_item (GTK_TOOLBAR (left_toolbar),
-			   _("ILS Directory"),
-			   _("Find friends on ILS"),
-			   NULL,
-			   gtk_image_new_from_stock (GM_STOCK_ILS_DIRECTORY, 
-						     GTK_ICON_SIZE_LARGE_TOOLBAR),
-			   GTK_SIGNAL_FUNC (gnomemeeting_component_view),
-			   (gpointer) gw->ldap_window); 
   
-  gtk_toolbar_append_item (GTK_TOOLBAR (left_toolbar),
-			   _("Chat"),
-			   _("Make a text chat with your friend"), 
-			   NULL,
-			   gtk_image_new_from_stock (GM_STOCK_TEXT_CHAT, 
-						     GTK_ICON_SIZE_LARGE_TOOLBAR),
-			   GTK_SIGNAL_FUNC (toolbar_button_changed),
-			   (gpointer) "/apps/gnomemeeting/view/show_chat_window");
-  
-  gtk_toolbar_append_item (GTK_TOOLBAR (left_toolbar),
-			   _("Control Panel"),
-			   _("Display the control panel"),
-			   NULL,
-			   gtk_image_new_from_stock (GM_STOCK_CONTROL_PANEL, 
-						     GTK_ICON_SIZE_LARGE_TOOLBAR),
-			   GTK_SIGNAL_FUNC (toolbar_cp_button_changed),
-			   (gpointer) "/apps/gnomemeeting/view/control_panel_section");
-
-  gtk_toolbar_append_space (GTK_TOOLBAR (left_toolbar));
-  
-
-  /* Both toolbars */
 
   /* The main horizontal toolbar */
   GtkWidget *toolbar = gtk_toolbar_new ();
@@ -276,13 +233,67 @@ void gnomemeeting_init_toolbar ()
                     G_CALLBACK (connect_button_clicked), 
 		    gw->connect_button);
 
-  gnome_app_add_docked (GNOME_APP (gm), hbox, "main_toolbar",
-  			BONOBO_DOCK_ITEM_BEH_EXCLUSIVE,
-  			BONOBO_DOCK_TOP, 1, 0, 0);
+  gtk_widget_show (GTK_WIDGET (gw->combo));
+  gtk_widget_show_all (GTK_WIDGET (gw->connect_button));
+  gtk_widget_show_all (GTK_WIDGET (hbox));
+  gtk_widget_show_all (GTK_WIDGET (toolbar));
 
-  gnome_app_add_toolbar (GNOME_APP (gm), GTK_TOOLBAR (left_toolbar),
- 			 "left_toolbar", BONOBO_DOCK_ITEM_BEH_EXCLUSIVE,
- 			 BONOBO_DOCK_LEFT, 3, 0, 0);
+  return hbox;
+}
+
+
+GtkWidget *gnomemeeting_init_left_toolbar (void)
+{
+  GtkWidget *image = NULL;
+  GtkWidget *left_toolbar = NULL;
+
+  gchar *device = NULL;
+  PString dev;
+  
+  GmWindow *gw = gnomemeeting_get_main_window (gm);
+
+  GConfClient *client = gconf_client_get_default ();
+
+  left_toolbar = gtk_toolbar_new ();
+  gtk_toolbar_set_orientation (GTK_TOOLBAR (left_toolbar), 
+			       GTK_ORIENTATION_VERTICAL);
+
+  image = gtk_image_new_from_stock (GM_STOCK_ILS_DIRECTORY, 
+				    GTK_ICON_SIZE_LARGE_TOOLBAR);
+  gtk_widget_show (image);
+  gtk_toolbar_append_item (GTK_TOOLBAR (left_toolbar),
+			   NULL,
+			   _("Open the address book"),
+			   NULL,
+			   image,
+			   GTK_SIGNAL_FUNC (gnomemeeting_component_view),
+			   (gpointer) gw->ldap_window); 
+
+
+  image =
+    gtk_image_new_from_stock (GM_STOCK_TEXT_CHAT, 
+			      GTK_ICON_SIZE_LARGE_TOOLBAR);
+  gtk_widget_show (image);
+  gtk_toolbar_append_item (GTK_TOOLBAR (left_toolbar),
+			   NULL,
+			   _("Make a text chat with your friend"), 
+			   NULL,
+			   image,
+			   GTK_SIGNAL_FUNC (toolbar_button_changed),
+			   (gpointer) "/apps/gnomemeeting/view/show_chat_window");
+  
+  image = gtk_image_new_from_stock (GM_STOCK_CONTROL_PANEL, 
+				    GTK_ICON_SIZE_LARGE_TOOLBAR);
+  gtk_widget_show (image);
+  gtk_toolbar_append_item (GTK_TOOLBAR (left_toolbar),
+			   NULL,
+			   _("Display the control panel"),
+			   NULL,
+			   image,
+			   GTK_SIGNAL_FUNC (toolbar_cp_button_changed),
+			   (gpointer) "/apps/gnomemeeting/view/control_panel_section");
+
+  gtk_toolbar_append_space (GTK_TOOLBAR (left_toolbar));
 
   gtk_toolbar_set_style (GTK_TOOLBAR (left_toolbar), GTK_TOOLBAR_ICONS);
  
@@ -304,11 +315,11 @@ void gnomemeeting_init_toolbar ()
 				gconf_client_get_bool (client, "/apps/gnomemeeting/devices/video_preview", NULL));
 
   g_signal_connect (G_OBJECT (gw->preview_button), "clicked",
-		    G_CALLBACK (toolbar_toggle_changed), 
+		    G_CALLBACK (toggle_changed), 
 		    (gpointer) "/apps/gnomemeeting/devices/video_preview");
 
   gtk_tooltips_set_tip (gw->tips, gw->preview_button,
-			_("Click here to begin to display images from your camera device."),
+			_("Click here to begin displaying images from your camera device."),
 			NULL);
 
   gtk_toolbar_append_widget (GTK_TOOLBAR (left_toolbar), 
@@ -353,7 +364,7 @@ void gnomemeeting_init_toolbar ()
 		    G_CALLBACK (pause_audio_callback), gw);
 
   gtk_tooltips_set_tip (gw->tips, gw->audio_chan_button,
-			_("Audio Transmission Status. During a call, click here to pause the audio transmission."), NULL);
+			_("Audio transmission status. During a call, click here to pause the audio transmission."), NULL);
 
   gtk_toolbar_append_widget (GTK_TOOLBAR (left_toolbar), 
 			     gw->audio_chan_button, NULL, NULL);
@@ -373,30 +384,28 @@ void gnomemeeting_init_toolbar ()
 		    G_CALLBACK (pause_video_callback), gw);
 
   gtk_tooltips_set_tip (gw->tips, gw->video_chan_button,
-			_("Video Transmission Status. During a call, click here to pause the video transmission."), NULL);
+			_("Video transmission status. During a call, click here to pause the video transmission."), NULL);
 
   gtk_toolbar_append_widget (GTK_TOOLBAR (left_toolbar), 
 			     gw->video_chan_button, NULL, NULL);
 
-
-  gtk_widget_show_all (GTK_WIDGET (hbox));
-
-  gtk_widget_show (GTK_WIDGET (gw->combo));
-  gtk_widget_show_all (GTK_WIDGET (gw->connect_button));
   gtk_widget_show_all (GTK_WIDGET (gw->preview_button));
 
 #ifdef HAS_IXJ
-  if (gconf_client_get_bool (client, "/apps/gnomemeeting/devices/lid", 0))
-    gtk_widget_show_all (GTK_WIDGET (gw->speaker_phone_button));
+  device =
+    gconf_client_get_string (client, DEVICES_KEY "audio_recorder", NULL);
+  if (device)
+    dev = PString (device);
+  //  if (dev.Find ("phone") != P_MAX_INDEX) 
+  // gtk_widget_show_all (GTK_WIDGET (gw->speaker_phone_button));
+  g_free (device);
 #endif
 
   gtk_widget_show_all (GTK_WIDGET (gw->audio_chan_button));
   gtk_widget_show_all (GTK_WIDGET (gw->video_chan_button));
 
-  if (gconf_client_get_bool (client, "/apps/gnomemeeting/view/left_toolbar", 0)) 
-    gtk_widget_show (GTK_WIDGET (gnome_app_get_dock_item_by_name(GNOME_APP (gm), "left_toolbar")));
-  else
-    gtk_widget_hide (GTK_WIDGET (gnome_app_get_dock_item_by_name(GNOME_APP (gm), "left_toolbar")));
+
+  return left_toolbar;
 }
 
 
