@@ -28,6 +28,7 @@
  *
  */
 
+
 #include "../config.h"
 
 #include "config.h"
@@ -44,6 +45,12 @@
 
 #include <sys/time.h>
 #include <gconf/gconf-client.h>
+#ifndef DISABLE_GNOME
+#include <gnome.h>
+#else
+#include <gtk/gtk.h>
+#endif
+
 
 /* Declarations */
 extern GnomeMeeting *MyApp;
@@ -59,6 +66,7 @@ GMVideoGrabber::GMVideoGrabber ()
 
   gw = gnomemeeting_get_main_window (gm);
   pw = gnomemeeting_get_pref_window (gm);
+  dw = gnomemeeting_get_druid_window (gm);
 
   /* Internal state */
   is_running = 1;
@@ -80,7 +88,6 @@ GMVideoGrabber::GMVideoGrabber ()
   video_channel = 0;
   video_size = 0;
   video_format = PVideoDevice::Auto;
-  tr_fps = 0;
   UpdateConfig ();
 
   /* Start the thread */
@@ -108,9 +115,10 @@ GMVideoGrabber::~GMVideoGrabber ()
   /* Enable the video preview buttons */
   gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button),
 			    TRUE);
-  gtk_widget_set_sensitive (GTK_WIDGET (pw->video_preview),
-			    TRUE);
-
+#ifndef DISABLE_GNOME
+  gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), true);
+#endif
+  
   /* Display the logo */
   gnomemeeting_init_main_window_logo (gw->main_video_image);
 
@@ -197,16 +205,15 @@ void GMVideoGrabber::Main ()
      that must be called just after the Main method exits. */
   gnomemeeting_threads_enter ();
   gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), FALSE);
-  gtk_widget_set_sensitive (GTK_WIDGET (pw->video_preview), FALSE);
+#ifndef DISABLE_GNOME
+  gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), FALSE);
+#endif
   gnomemeeting_threads_leave ();
   
   /* if opened, we close */
   if (IsOpened ()) {
 
-    grabber->Close ();
-    channel->Close ();
     Current ()->Sleep (500);
-
     delete (channel);
   }
 
@@ -228,12 +235,6 @@ void GMVideoGrabber::UpdateConfig ()
   video_size =  gconf_client_get_int (GCONF_CLIENT (client), "/apps/gnomemeeting/devices/video_size", NULL);
 
 
-  /* The number of Transmitted FPS must be equal to 0 to disable */
-  if (!gconf_client_get_bool (client, "/apps/gnomemeeting/video_settings/enable_fps", NULL))
-    tr_fps = 0;
-  else
-    tr_fps = gconf_client_get_int (client, "/apps/gnomemeeting/video_settings/tr_fps", NULL);
-
   switch (gconf_client_get_int (GCONF_CLIENT (client), "/apps/gnomemeeting/devices/video_format", NULL)) {
     
   case 0:
@@ -456,8 +457,10 @@ void GMVideoGrabber::VGOpen (void)
     /* Disable the video preview button while opening */
     gnomemeeting_threads_enter ();
     gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), FALSE);
-    gtk_widget_set_sensitive (GTK_WIDGET (pw->video_preview), FALSE);
-    gnomemeeting_statusbar_flash (gm, _("Opening Video device"));
+#ifndef DISABLE_GNOME
+    gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), FALSE);
+#endif
+    gnomemeeting_statusbar_flash (gw->statusbar, _("Opening Video device"));
     gnomemeeting_log_insert (gw->history_text_view, _("Opening Video device"));
     gnomemeeting_threads_leave ();
     
@@ -493,7 +496,7 @@ void GMVideoGrabber::VGOpen (void)
 	if (!grabber->SetColourFormatConverter ("YUV420P"))
 	  error_code = 3;
       else
-	if (!grabber->SetFrameRate (tr_fps))
+	if (!grabber->SetFrameRate (30))
 	  error_code = 4;
       else
 	if (!grabber->SetFrameSizeConverter (width, height, FALSE))
@@ -509,7 +512,7 @@ void GMVideoGrabber::VGOpen (void)
 	(_("Successfully opened video device %s, channel %d"), 
 	 video_device, video_channel);
       gnomemeeting_log_insert (gw->history_text_view, msg);
-      gnomemeeting_statusbar_flash (gm, _("Video Device Opened"));
+      gnomemeeting_statusbar_flash (gw->statusbar, _("Video Device Opened"));
       g_free (msg);
       gnomemeeting_threads_leave ();
     }
@@ -523,7 +526,7 @@ void GMVideoGrabber::VGOpen (void)
 	gnomemeeting_threads_enter ();
 	msg = g_strdup_printf 
 	  (_("Error while opening video device %s, channel %d.\nThe chosen Video Image will be transmitted during calls. If you didn't choose any image, then the default GnomeMeeting logo will be transmitted. Notice that you can always transmit a given image or the GnomeMeeting logo by choosing \"Picture\" as video device."), video_device, video_channel);
-	gnomemeeting_statusbar_flash (gm, _("Can't open the Video Device"));
+	gnomemeeting_statusbar_flash (gw->statusbar, _("Can't open the Video Device"));
 	
 	switch (error_code)	{
 	  
@@ -572,6 +575,12 @@ void GMVideoGrabber::VGOpen (void)
       grabber->SetFrameRate (6);
       grabber->SetFrameSizeConverter (width, height, FALSE);
 
+      gnomemeeting_threads_enter ();
+#ifndef DISABLE_GNOME
+      gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), true);
+#endif
+      gnomemeeting_threads_leave ();
+      
       g_free (video_image);
     }
 
@@ -610,10 +619,7 @@ void GMVideoGrabber::VGOpen (void)
     if (MyApp->Endpoint ()->GetCallingState () == 0) {
 
       gnomemeeting_threads_enter ();      
-      gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button),
-				TRUE);
-      gtk_widget_set_sensitive (GTK_WIDGET (pw->video_preview),
-				TRUE);
+      gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), true);
       gnomemeeting_threads_leave ();
     }
 
@@ -654,7 +660,9 @@ void GMVideoGrabber::VGClose (int display_logo)
     /* Disable the video preview button while closing */
     gnomemeeting_threads_enter ();
     gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), FALSE);
-    gtk_widget_set_sensitive (GTK_WIDGET (pw->video_preview), FALSE);
+#ifndef DISABLE_GNOME
+    gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), FALSE);
+#endif
     gnomemeeting_threads_leave ();
     
     if (channel) 
@@ -670,14 +678,16 @@ void GMVideoGrabber::VGClose (int display_logo)
   
     /* Enable the video preview button  */
     gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), TRUE);
-    gtk_widget_set_sensitive (GTK_WIDGET (pw->video_preview), TRUE);
+#ifndef DISABLE_GNOME
+    gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), TRUE);
+#endif
     
     /* Display the logo again */
     if (display_logo)
       gnomemeeting_init_main_window_logo (gw->main_video_image);
 
 
-    gnomemeeting_statusbar_flash (gm, _("Video Device Closed"));
+    gnomemeeting_statusbar_flash (gw->statusbar, _("Video Device Closed"));
     
     /* Disable the video settings frame */
     gtk_widget_set_sensitive (GTK_WIDGET (gw->video_settings_frame),
@@ -707,7 +717,154 @@ void GMVideoGrabber::VGClose (int display_logo)
      is really closed */
   PThread::Current ()->Sleep (1000);
 
-  device_mutex.Signal ();
   grabbing_mutex.Signal ();
+  device_mutex.Signal ();
 }
 
+
+/* The video tester class */
+GMVideoTester::GMVideoTester ()
+  :PThread (1000, AutoDeleteThread)
+{
+#ifndef DISABLE_GNOME
+
+  this->Resume ();
+#endif
+}
+
+
+GMVideoTester::~GMVideoTester ()
+{
+#ifndef DISABLE_GNOME
+  quit_mutex.Wait ();
+
+  quit_mutex.Signal ();
+ #endif
+}
+
+
+void GMVideoTester::Main ()
+{
+#ifndef DISABLE_GNOME
+  quit_mutex.Wait ();
+
+  GmWindow *gw = NULL;
+  GmDruidWindow *dw = NULL;
+  
+  PVideoInputDevice *grabber = new PVideoInputDevice();
+  
+  int height = GM_QCIF_HEIGHT; 
+  int width = GM_QCIF_WIDTH; 
+  int error_code = -1;
+  int cpt = 0;
+  gdouble per = 0.0;
+
+  gchar *msg = NULL;
+  gchar *video_device = NULL;
+
+  GConfClient *client = gconf_client_get_default ();
+
+  video_device =  gconf_client_get_string (GCONF_CLIENT (client), "/apps/gnomemeeting/devices/video_recorder", NULL);
+
+  gdk_threads_enter ();
+  gw = gnomemeeting_get_main_window (gm);
+  dw = gnomemeeting_get_druid_window (gm);
+  gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), FALSE);
+  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (dw->progress), "");
+  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (dw->progress), 0.0);
+  gdk_threads_leave ();
+
+  
+  while (cpt <= 3) {
+
+    gdk_threads_enter ();
+    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (dw->progress), per);
+    gtk_widget_queue_draw (GTK_WIDGET (dw->progress));
+    gdk_threads_leave ();
+    
+    if (strcmp (video_device, _("Picture"))) {
+      
+      if (!grabber->Open (video_device, FALSE))
+	error_code = 0;
+      else
+	if (!grabber->SetVideoChannelFormat (0,  PVideoDevice::Auto))
+	  error_code = 2;
+	else
+	  if (!grabber->SetColourFormatConverter ("YUV420P"))
+	    error_code = 3;
+	  else
+	    if (!grabber->SetFrameRate (10))
+	      error_code = 4;
+	    else
+	      if (!grabber->SetFrameSizeConverter (width, height, FALSE))
+		error_code = 5;
+	      else
+		grabber->Close ();
+
+      if (error_code != -1)
+	break;
+    }
+    
+    msg = g_strdup_printf (_("Test %d done"), cpt);
+    gdk_threads_enter ();
+    gtk_progress_bar_set_text (GTK_PROGRESS_BAR (dw->progress), msg);
+    gdk_threads_leave ();
+    g_free (msg);
+
+    per = cpt * 0.33;
+    cpt++;
+    PThread::Current () ->Sleep (100);
+  }
+
+
+  if (error_code != - 1) {
+    
+    switch (error_code)	{
+	  
+    case 0:
+      msg = g_strdup_printf (_("Error while opening %s."), video_device);
+      break;
+      
+    case 1:
+      msg = g_strdup_printf (_("Your video driver doesn't support the requested video format."));
+      break;
+      
+    case 2:
+      msg = g_strdup_printf (_("Could not open the chosen channel with the chosen video format."));
+      break;
+      
+    case 3:
+      msg = g_strdup_printf (_("Your driver doesn't support the YUV420P color format"));
+      break;
+      
+    case 4:
+      msg = g_strdup_printf ( _("Error with the frame rate."));
+      break;
+      
+    case 5:
+      msg = g_strdup_printf (_("Error with the frame size."));
+      break;
+    }
+  }  
+  else
+    msg = g_strdup (_("Tests OK!"));
+
+  gdk_threads_enter ();
+  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (dw->progress), msg);
+  gdk_threads_leave ();
+  g_free (msg);
+
+  delete (grabber);
+
+  gdk_threads_enter ();
+  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (dw->progress), 1.0);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dw->video_test_button),
+				FALSE);
+  gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), TRUE);
+  gdk_threads_leave ();
+
+  quit_mutex.Signal ();
+#endif
+}
+
+
